<!DOCTYPE html>
<html lang="pt-br">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>MONITORAMENTO - <?php echo $_REQUEST['identificacao']; ?></title>

    <style>
        html, body, #map, #container {
            height: 100%;
            margin: 0;
            padding: 0;
            font-family: Arial, sans-serif;
            background-color: black;
        }

        #container {
            display: flex;
            flex-direction: row;
            height: 100%;
            overflow: hidden;
        }

        #map {
            flex: 1;
            border-right: 2px solid #ccc;
            min-height: 50%;
        }

        #dados {
            flex: 1;
            padding: 0px;
            display: flex;
            flex-direction: column;
            justify-content: space-between;
            color: #f9f9f9;
        }

        #info {
            background: rgba(51, 51, 51, 0.8);
            padding: 7px;
            border-radius: 8px;
            overflow-y: auto;
            margin-bottom: 20px;
            box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
            flex-grow: 1;
        }

        table {
            width: 100%;
            border-collapse: collapse;
            margin-bottom:7px;
            border: 1px solid #555;
        }

        th, td {
            padding: 7px;
            text-align: left;
            border: 1px solid #555;
        }

        th {
            background: #444;
            color: #fff;
        }

        tr:nth-child(even) {
            background: #444;
        }

        tr:nth-child(odd) {
            background: #555;
        }

        #weather-table-horizontal {
            width: 100%;
            border-collapse: collapse;
            margin-top: 0px;
            border: 1px solid #555;
        }

        #weather-table-horizontal th, #weather-table-horizontal td {
            padding: 7px;
            text-align: center;
            border: 1px solid #555;
        }

        #weather-table-horizontal th {
            background: #444;
            color: #fff;
        }

        #weather-table-horizontal tr:nth-child(even) {
            background: #444;
        }

        #weather-table-horizontal tr:nth-child(odd) {
            background: #555;
        }

        @media (max-width: 800px) {
            #container {
                flex-direction: column;
            }

            #map {
                order: 2;
                height: 100%;
                border-right: none;
                border-bottom: 2px solid #ccc;
            }

            #dados {
                order: 1;
                position: absolute;
                top: 0;
                left: 0;
                right: 0;
                z-index: 1;
                padding: 1px;
                margin: 1px;
                height: auto;
            }

            #info {
                background: rgba(51, 51, 51, 0.8);

            }

            button {
                width: 100%;
                box-sizing: border-box;
            }

            .hidden {
                display: none;
            }
        }
    </style>

    <script>
        let map;
        let route;
        let boatOverlay;

        function initialize() {
            const mapOptions = {
                zoom: 13,
                mapTypeId: 'hybrid',
                streetViewControl: false,
                fullscreenControl: false,
                mapTypeControl: false
            };
            map = new google.maps.Map(document.getElementById("map"), mapOptions);

            route = new google.maps.Polyline({
                path: [],
                geodesic: true,
                strokeColor: '#ccc',
                strokeOpacity: 0.7,
                strokeWeight: 6,
                editable: false,
                icons: [{
                    icon: {
                        path: google.maps.SymbolPath.CIRCLE,
                        fillColor: '#D8C1A5',
                        fillOpacity: 0.7,
                        scale: 2,
                        strokeColor: '#ccc',
                        strokeOpacity: 0.7,
                        strokeWeight: 1
                    },
                    offset: '0',
                    repeat: '20px'
                }],
                map: map
            });

            function BoatOverlay(position, map) {
                this.position = position;
                this.div = null;
                this.setMap(map);
            }

            BoatOverlay.prototype = new google.maps.OverlayView();

            BoatOverlay.prototype.onAdd = function() {
                const div = document.createElement('div');
                div.style.position = 'absolute';
                div.style.width = '45px';
                div.style.height = '45px';
                div.style.backgroundImage = 'url(https://enave.com.br/n.png)';
                div.style.backgroundSize = 'contain';
                this.div = div;

                const panes = this.getPanes();
                panes.overlayImage.appendChild(div);
            };

            BoatOverlay.prototype.draw = function() {
                if (!this.div) return;

                const overlayProjection = this.getProjection();
                const position = overlayProjection.fromLatLngToDivPixel(this.position);

                const div = this.div;
                div.style.left = position.x - 23 + 'px';
                div.style.top = position.y - 23 + 'px';
            };

            BoatOverlay.prototype.setPosition = function(position) {
                this.position = position;
                if (this.div) {
                    this.draw();
                }
            };

            BoatOverlay.prototype.setRotation = function(rotation) {
                if (this.div) {
                    this.div.style.transform = 'rotate(' + rotation + 'deg)';
                }
            };

            BoatOverlay.prototype.onRemove = function() {
                if (this.div) {
                    this.div.parentNode.removeChild(this.div);
                    this.div = null;
                }
            };

            boatOverlay = new BoatOverlay(new google.maps.LatLng(-3.1190275, -60.0217314), map);

            fetchDataAndUpdateMap(true); // Initial call with centering
        }

        function fetchDataAndUpdateMap(initialLoad = false) {
            const identificacao = '<?php echo $_REQUEST['identificacao']; ?>';
            fetch('/wp-admin/admin-ajax.php?action=pegar_dados_mapa&identificacao=' + identificacao)
                .then(response => response.json())
                .then(data => {
                    if (data.pathCoords) {
                        updateMap(data.pathCoords, initialLoad);
                    }
                })
                .catch(error => console.error('Error fetching map data:', error));

            fetch('/wp-admin/admin-ajax.php?action=pegar_dados_info&identificacao=' + identificacao)
                .then(response => response.json())
                .then(data => {
                    if (data.error) {
                        console.error('Error:', data.error);
                    } else {
                        updateInfo(data);
                    }
                })
                .catch(error => console.error('Error fetching info data:', error));
        }

        function updateMap(pathCoords, initialLoad) {
            if (pathCoords.length < 2) {
                console.error('Not enough data points to determine direction.');
                return;
            }

            const lastCoord = pathCoords[pathCoords.length - 1];
            const secondLastCoord = pathCoords[pathCoords.length - 2];

            const latLngs = pathCoords.map(coord => new google.maps.LatLng(coord.lat, coord.lng));
            route.setPath(latLngs);

            const lastLatLng = new google.maps.LatLng(lastCoord.lat, lastCoord.lng);
            const secondLastLatLng = new google.maps.LatLng(secondLastCoord.lat, secondLastCoord.lng);

            const heading = google.maps.geometry.spherical.computeHeading(secondLastLatLng, lastLatLng);

            boatOverlay.setPosition(lastLatLng);
            boatOverlay.setRotation(heading);

            if (initialLoad) {
                map.setCenter(lastLatLng); // Center the map on the initial load
            }
        }

        function updateInfo(data) {
            const infoTable = document.getElementById('info-table');
            infoTable.innerHTML = ''; // Clear previous data

            const now = new Date().getTime() / 1000;
            const dataHoraAtualizada = data.dataHoraAtualizada;
            const corAtualizado = data.corAtualizado;
            const titulo_texto = data.titulo_texto;

            let html = '';
            html += `<tr style="color:${corAtualizado}"><td>${titulo_texto}</td><td>${new Date(dataHoraAtualizada * 1000).toLocaleString()}</td></tr>`;
            html += `<tr><td><i class="fas fa-tachometer-alt"></i> Speed Atual </td><td>${data.velocidadeAtual} Km/h </td></tr>`;
            html += `<tr><td><i class="fas fa-tachometer-alt"></i> Speed M&eacute;dia </td><td>${data.velocidadeMedia} Km/h </td></tr>`;

            infoTable.innerHTML = html;

            // Update the arrival predictions if applicable
            if (data.previsoes) {
                updateArrivalPredictions(data.previsoes);
            }
        }

        function updateArrivalPredictions(previsoes) {
            const horarioTable = document.getElementById('horario-table');
            let html = '';

            previsoes.forEach(escala => {
                html += `<tr class="hidden"><td><i class="fas fa-map-marker-alt"></i> Previs&atilde;o - ${escala.nome}</td><td>${escala.horarioChegada}</td></tr>`;
            });

            horarioTable.innerHTML = html;
            alternateRows();
        }

        document.addEventListener('DOMContentLoaded', function() {
            document.getElementById('refreshData').addEventListener('click', () => fetchDataAndUpdateMap(false));
        });

        function alternateRows() {
            const rows = document.querySelectorAll('#horario-table tr');
            let currentIndex = 0;

            function showNextRow() {
                if (rows.length > 0) {
                    rows.forEach((row, index) => {
                        if (rows.length === 1) {
                            row.classList.remove('hidden');
                        } else {
                            if (index === currentIndex) {
                                row.classList.remove('hidden');
                                setTimeout(() => {
                                    row.classList.add('hidden');
                                }, 10000);
                            } else {
                                row.classList.add('hidden');
                            }
                        }
                    });
                    if (rows.length > 1) {
                        currentIndex++;
                        if (currentIndex >= rows.length) {
                            currentIndex = 0;
                        }
                    }
                }
            }

            setInterval(showNextRow, 8000);
            showNextRow();
        }

        window.addEventListener('load', function() {
            if (window.innerWidth <= 800) {
                alternateRows();
            }

            // Initialize map
            initialize();

            // Start periodic updates
            setInterval(() => fetchDataAndUpdateMap(false), 10000); // Update every 10 seconds
        });
    </script>
</head>
<body>
    <div id="container">
        <?php
        date_default_timezone_set('America/Manaus');
        ini_set('display_errors', 0);
        $identificacao = $_REQUEST['identificacao'];

        // Obtendo os dados meteorolgicos
        function getWeatherData($latitude, $longitude, $api_key) {
            $url = "http://api.openweathermap.org/data/2.5/weather?lat={$latitude}&lon={$longitude}&appid={$api_key}&units=metric&lang=pt_br";
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            $resposta = curl_exec($ch);
            curl_close($ch);
            if ($resposta) {
                return json_decode($resposta, true);
            }
            return null;
        }

        // Coordenadas de exemplo
        $latitude = '-4.234243';
        $longitude = '-69.945610';
        $api_key = '473df009aab8d20f9ae589f203a410f1';

        $weather_data = getWeatherData($latitude, $longitude, $api_key);

        // Verificando se os dados so recentes
        $args = array(
            'post_type' => 'dados_lancha',
            'posts_per_page' => 1,
            'order' => 'DESC',
            'meta_query' => array(
                array(
                    'key' => 'identificacao',
                    'value' => $identificacao,
                    'compare' => '=',
                ),
            ),
        );

        $qry = new WP_QUERY($args);
        $post = $qry->get_posts()[0];

        if ($post) {
            setup_postdata($post);

            $last_update_time = strtotime(get_post_meta($post->ID, 'datah', true));
            $now = time();
            $is_data_recent = ($now - $last_update_time) <= 1800;

            wp_reset_postdata();
        } else {
            $is_data_recent = false;
        }
        ?>

        <?php if ($is_data_recent): ?>
            <div id="map"></div>
        <?php else: ?>
            <div id="map" style="display: flex; align-items: center; justify-content: center;">
                <h2 style="color: red; margin: 10px;">Desligado..</h2>
            </div>
        <?php endif; ?>

        <div id="dados">
            <div id="info">
               <button id="refreshData" style="
    padding: 7px 20px;
    font-size: 16px;
    cursor: pointer;
    background-color: #3C87D0;
    color: #fff;
    border: none;
    border-radius: 5px;
    transition: background-color 0.3s;
    margin: 0px 0px 5px 0px;
    width: 100%;
    text-transform: uppercase;
    letter-spacing: 2px;
    font-weight: bold;
    text-align: center;
    text-shadow: 1px 1px 2px rgba(0, 0, 0, 0.5); /* Sombra no texto */
"><?php echo $identificacao;?></button>

                <table id="info-table"></table>
                <table id="horario-table"></table>

                <!-- Nova Tabela para Clima Atual -->
                <?php if ($weather_data): ?>
                    <table id="weather-table-horizontal">
                        <thead>
                            <tr>
                                <th>Temperatura</th>
                                <th>Nuvens</th>
                                <th>Umidade</th>
                            </tr>
                        </thead>
                        <tbody>
                            <tr>
                                <td><?php echo $weather_data['main']['temp']; ?>&deg;C</td>
                                <td><?php echo ucfirst($weather_data['weather'][0]['description']); ?></td>
                                <td><?php echo $weather_data['main']['humidity']; ?>%</td>
                            </tr>
                        </tbody>
                    </table>
                <?php endif; ?>
            </div>
        </div>
    </div>
    <?php if ($is_data_recent): ?>
        <script src="https://maps.googleapis.com/maps/api/js?key=AIzaSyAEVO5HtNe7wbg7ao1E68Q6UMM2TmSjk7I&callback=initialize&libraries=geometry" async defer></script>
    <?php endif; ?>
</body>
</html>
