<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>CesiumJS Barco com Rastro Contnuo</title>
    <script src="https://cesium.com/downloads/cesiumjs/releases/1.88/Build/Cesium/Cesium.js"></script>
    <link href="https://cesium.com/downloads/cesiumjs/releases/1.88/Build/Cesium/Widgets/widgets.css" rel="stylesheet">
    <style>
        html, body, #cesiumContainer {
            width: 100%;
            height: 100%;
            margin: 0;
            padding: 0;
            overflow: hidden;
        }
    </style>
</head>
<body>
    <div id="cesiumContainer"></div>

    <script>
    Cesium.Ion.defaultAccessToken = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqdGkiOiI1YTI0YTI5Yi1hYmZhLTRlZjQtYmRhOS0xZDYzN2Q2YTE0YjAiLCJpZCI6MjM3MjA0LCJpYXQiOjE3MjQ3MzExNDF9.uyOaFwq9ccT8IX27raWwIs_Fc6glz_UVuTMnogOvszc';

    var viewer = new Cesium.Viewer('cesiumContainer', {
        terrainProvider: Cesium.createWorldTerrain(),
        timeline: false,
        animation: true,
        sceneMode: Cesium.SceneMode.SCENE3D
    });

    var boatEntity = null;
    var polylinePositions = [];
    var polylineEntity = null;
    var lastCoord = null;
    var moveBoatInterval = 5000;
    var duration = 5000;  // Durao da animao entre coordenadas

    // Funo para buscar dados GPS e ordenar pelas datas mais recentes
    function fetchGPSData() {
        console.log('Fetching GPS data...');
        return fetch('https://enave.com.br/wp-json/lancha/v1/dados-lancha/?identificacao=<?php echo $_REQUEST["identificacao"];?>')
            .then(response => {
                if (!response.ok) {
                    console.error('Erro ao buscar dados da API:', response.status);
                    return null;
                }
                return response.json();
            })
            .then(data => {
                console.log('Dados recebidos:', data);
                if (data.length >= 10) {
                    // Ordena as coordenadas pelas datas mais recentes
                    var sortedData = data.sort((a, b) => new Date(b.data) - new Date(a.data));

                    // Pega as 10 coordenadas mais recentes
                    var recentCoords = sortedData.slice(0, 10).map(item => ({
                        lat: parseFloat(item.latitude),
                        lon: parseFloat(item.longitude),
                        data: item.data  // Mantemos a data para verificao
                    }));
                    console.log('Coordenadas organizadas pelas datas:', recentCoords);
                    return recentCoords;
                } else {
                    console.error('Coordenadas insuficientes recebidas:', data);
                    return null;
                }
            })
            .catch(error => console.error('Erro ao buscar dados da API:', error));
    }

    // Funo para calcular o bearing entre duas coordenadas
    function calculateBearing(start, end) {
        var startLat = Cesium.Math.toRadians(start.lat);
        var startLon = Cesium.Math.toRadians(start.lon);
        var endLat = Cesium.Math.toRadians(end.lat);
        var endLon = Cesium.Math.toRadians(end.lon);

        var dLon = endLon - startLon;
        var y = Math.sin(dLon) * Math.cos(endLat);
        var x = Math.cos(startLat) * Math.sin(endLat) - Math.sin(startLat) * Math.cos(endLat) * Math.cos(dLon);
        var bearing = Math.atan2(y, x);
        return Cesium.Math.toDegrees(bearing);
    }

    // Funo para reorganizar a ordem das coordenadas
    function rearrangeCoordinates(coords) {
        // Organizando na ordem: 10, 9, 7, 6, 5, 4, 3, 2, 1
        var rearranged = [
            coords[9],  // 10
            coords[8],  // 9
            coords[6],  // 7
            coords[5],  // 6
            coords[4],  // 5
            coords[3],  // 4
            coords[2],  // 3
            coords[1],  // 2
            coords[0]   // 1
        ];
        return rearranged;
    }

   // Funo para inicializar o barco e o rastro
function initializeBoat(coords) {
    if (coords.length >= 10) {
        var rearrangedCoords = rearrangeCoordinates(coords);

        var lastPosition = rearrangedCoords[rearrangedCoords.length - 1];  // ltima coordenada (onde o barco estar)
        var secondLastPosition = rearrangedCoords[rearrangedCoords.length - 2];  // Penltima coordenada para calcular o bearing

        var bearing = calculateBearing(lastPosition, secondLastPosition);

        console.log('Inicializando barco na posio:', lastPosition);

        // Centraliza a cmera na ltima posio (apenas na inicializao)
        viewer.camera.flyTo({
            destination: Cesium.Cartesian3.fromDegrees(lastPosition.lon, lastPosition.lat, 5000),
            duration: 2,
            orientation: {
                heading: Cesium.Math.toRadians(bearing + 180),  // Ajuste para que a cmera olhe para frente, no rumo da linha
                pitch: Cesium.Math.toRadians(-45),  // ngulo da cmera para o cho
                roll: 0.0
            }
        });

        // Inicializa o barco na ltima posio, ajustando a rotao 180 graus
        boatEntity = viewer.entities.add({
            position: Cesium.Cartesian3.fromDegrees(lastPosition.lon, lastPosition.lat, 0),
            billboard: {
                image: 'https://enave.com.br/n.png',  // Verifique se a URL est correta
                width: 45,
                height: 45,
                verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
                rotation: Cesium.Math.toRadians(90),  // Inverter o cone em 180 graus
                alignedAxis: Cesium.Cartesian3.UNIT_Z  // Mantm o cone alinhado com o eixo Z (para no sair da posio)
            },
            orientation: Cesium.Transforms.headingPitchRollQuaternion(
                Cesium.Cartesian3.fromDegrees(lastPosition.lon, lastPosition.lat, 0),
                new Cesium.HeadingPitchRoll(
                    Cesium.Math.toRadians(bearing),  // Ajuste do heading
                    0,  // Pitch
                    0   // Roll
                )
            )
        });

        if (!boatEntity) {
            console.error('Erro ao adicionar o barco  cena.');
        }

        // Cria o rastro do barco com as coordenadas reorganizadas
        polylinePositions = rearrangedCoords.map(coord => Cesium.Cartesian3.fromDegrees(coord.lon, coord.lat, 0));
        polylineEntity = viewer.entities.add({
            polyline: {
                positions: polylinePositions,
                width: 10,
                material: Cesium.Color.YELLOW.withAlpha(0.7)
            }
        });

        if (!polylineEntity) {
            console.error('Erro ao adicionar o rastro  cena.');
        }

        lastCoord = lastPosition;

        // Adiciona o zoom-out 3 vezes, aps 3 segundos da inicializao
        setTimeout(function() {
            zoomOutEffect();
        }, 3000);  // Atraso de 3 segundos
    } else {
        console.error('Coordenadas insuficientes para inicializar o barco:', coords);
    }
}

// Funo para aplicar o efeito de zoom-out 3 vezes
function zoomOutEffect() {
    var zoomOutTimes = 10;  // Nmero de vezes para dar zoom-out
    var zoomOutInterval = 10;  // Intervalo entre cada zoom-out (em milissegundos)

    function performZoomOut(times) {
        if (times > 0) {
            viewer.camera.zoomOut(3000);  // Aumenta o zoom-out
            setTimeout(function() {
                performZoomOut(times - 1);  // Chama recursivamente at completar o nmero de zooms
            }, zoomOutInterval);  // Tempo entre os zooms
        }
    }

    // Inicia o zoom-out
    performZoomOut(zoomOutTimes);
}

    // Funo para buscar dados e animar o barco
    // Funo para buscar dados e animar o barco
function fetchAndAnimate() {
    fetchGPSData().then(newCoords => {
        if (newCoords && newCoords.length > 0) {
            var nextCoord = newCoords[0];  // Pega a nova coordenada mais recente
            console.log('Nova coordenada recebida:', nextCoord);

            if (!lastCoord || (nextCoord.lat !== lastCoord.lat || nextCoord.lon !== lastCoord.lon)) {
                var previousCoord = lastCoord;
                lastCoord = nextCoord;
                animateBoat(previousCoord, nextCoord);
            }
        } else {
            console.error('Nenhuma nova coordenada recebida.');
        }

        // Continuar a buscar novas coordenadas aps o intervalo definido
        setTimeout(fetchAndAnimate, moveBoatInterval);
    });
}

// Funo para animar o movimento do barco e continuar o rastro
function animateBoat(previousCoord, newCoord) {
    var startTime = Date.now();

    function moveBoat() {
        var currentTime = Date.now();
        var timeElapsed = currentTime - startTime;
        var t = timeElapsed / duration;
        if (t >= 1) t = 1;

        var interpolatedLon = Cesium.Math.lerp(previousCoord.lon, newCoord.lon, t);
        var interpolatedLat = Cesium.Math.lerp(previousCoord.lat, newCoord.lat, t);
        var newPosition = Cesium.Cartesian3.fromDegrees(interpolatedLon, interpolatedLat, 0);

        // Atualiza a posio do barco
        boatEntity.position = newPosition;

        // Adiciona a nova posio ao rastro
        polylinePositions.push(newPosition);
        polylineEntity.polyline.positions = polylinePositions;

        var bearing = calculateBearing(previousCoord, newCoord);
        boatEntity.orientation = Cesium.Transforms.headingPitchRollQuaternion(
            newPosition,
            new Cesium.HeadingPitchRoll(
                Cesium.Math.toRadians(bearing),  // Ajuste do heading para alinhar o barco com a linha
                0,  // Pitch
                0   // Roll
            )
        );

        if (t >= 1) {
            // Movimentao finalizada
        } else {
            requestAnimationFrame(moveBoat);
        }
    }

    requestAnimationFrame(moveBoat);
}

// Inicializa o barco e a animao
fetchGPSData().then(coords => {
    if (coords && coords.length > 0) {
        initializeBoat(coords);
        setTimeout(fetchAndAnimate, moveBoatInterval);
    } else {
        console.error('Erro ao inicializar barco. Coordenadas invlidas:', coords);
    }
});

    </script>
</body>
</html>
