<!DOCTYPE html>
<html lang="pt-BR">
<head>
  <meta charset="UTF-8">
  <title>ENAVE 3D</title>
  <script src="https://cesium.com/downloads/cesiumjs/releases/1.111/Build/Cesium/Cesium.js"></script>
  <link href="https://cesium.com/downloads/cesiumjs/releases/1.111/Build/Cesium/Widgets/widgets.css" rel="stylesheet">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <style>
    html, body, #cesiumContainer {
      width: 100%;
      height: 100%;
      margin: 0;
      padding: 0;
      overflow: hidden;
    }
    .cesium-viewer-bottom {
      display: none !important;
    }
  </style>
</head>
<body>
  <div id="cesiumContainer"></div>
  <script>
    (async () => {
      // Adicionando o token do Cesium Ion
      Cesium.Ion.defaultAccessToken = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqdGkiOiI5NzI3MDI2NC05OTlmLTQwMmItOTU5OS05ZDU0MmNiNjdkZDkiLCJpZCI6MjM3MjA0LCJpYXQiOjE3MjUyMTMyNjJ9.WYtyRVGqpGmRuApRHow5epcYJJX-O37AdYNxvRZ8gcA';

      // Criando o visualizador do Cesium
      const viewer = new Cesium.Viewer('cesiumContainer', {
        shouldAnimate: false,
        animation: false,
        timeline: false,
        baseLayerPicker: false,
        geocoder: false,
        sceneModePicker: false,
        infoBox: false,
        homeButton: false,
        navigationHelpButton: false,
        fullscreenButton: false,
        globe: false, // Mantenha como false se funcionar melhor para voc
      });

      // Habilitar a atmosfera do cu para visualizao
      viewer.scene.skyAtmosphere.show = true;

      // Define o tempo fixo para 12h (meio-dia)
      const clock = viewer.clock;
      clock.currentTime = Cesium.JulianDate.fromDate(new Date(2023, 6, 21, 12)); // Define o tempo para 12h
      clock.shouldAnimate = true; // Permite o movimento

      // Adicionar Photorealistic 3D Tiles do Google
      try {
        const tileset = await Cesium.createGooglePhotorealistic3DTileset();
        viewer.scene.primitives.add(tileset);
      } catch (error) {
        console.error('Erro ao carregar o tileset Photorealistic 3D Tiles:', error);
      }

      // Lista de barcos
      const boatNames = [
        'GLORIA DE DEUS 3',
        'LEAO DE JUDA V',
        'GLORIA DE DEUS 7',
        'Ajato 2000',
        'F/B FREI GALVAO',
        //'Belissima',
        'F/B MARIA MONTEIRO',
        'F/B M MONTEIRO 2',
        //'ThaisHolanda',
        'MOREIRA JR',
        'Madame Crys',
      ];

      const defaultCoord = { lat: -23.5505, lon: -46.6333 }; // Posio padro (So Paulo)

      let vehicleEntities = {};
      let polylineEntities = {};
      let polylinePositions = {}; // Posies para cada barco
      let lastProcessedCoords = {}; // ltima coordenada processada para cada barco
      let isMoving = {}; // Controle de movimento para cada barco
      let isFirstBoatInitialized = false; // Controle para saber se o primeiro barco j foi inicializado

      // Funo para validar coordenadas
      function isValidCoord(coord) {
        return coord && !isNaN(coord.lat) && !isNaN(coord.lon);
      }

      // Funo para calcular o heading entre duas coordenadas
      function calculateHeading(fromCoord, toCoord) {
        try {
          if (!isValidCoord(fromCoord) || !isValidCoord(toCoord)) {
            console.warn('Coordenadas invlidas para clculo de heading');
            return 0;
          }

          const fromPosition = Cesium.Cartesian3.fromDegrees(
            fromCoord.lon,
            fromCoord.lat
          );
          const toPosition = Cesium.Cartesian3.fromDegrees(
            toCoord.lon,
            toCoord.lat
          );
          const direction = Cesium.Cartesian3.subtract(
            toPosition,
            fromPosition,
            new Cesium.Cartesian3()
          );
          Cesium.Cartesian3.normalize(direction, direction);

          let headingRadians = Math.atan2(direction.y, direction.x);
          if (isNaN(headingRadians)) {
            console.warn('Erro ao calcular heading: resultado  NaN');
            return 0;
          }
          return Cesium.Math.toDegrees(headingRadians);
        } catch (error) {
          console.warn('Erro ao calcular o heading:', error);
          return 0;
        }
      }

      // Funo para mover o barco suavemente entre as posies
      function moveBoat(previousCoord, newCoord, boatName) {
        try {
          if (!isValidCoord(previousCoord) || !isValidCoord(newCoord)) {
            console.warn(`Coordenadas invlidas para o barco: ${boatName}`);
            return;
          }

          let startTime = Date.now();
          let duration = 10000; // Durao da animao (10 segundos)
          let startPosition = Cesium.Cartesian3.fromDegrees(previousCoord.lon, previousCoord.lat);
          let endPosition = Cesium.Cartesian3.fromDegrees(newCoord.lon, newCoord.lat);

          if (!startPosition || !endPosition) {
            console.warn(`Posies invlidas para o barco: ${boatName}`);
            return;
          }

          // Calcular o heading dinamicamente com base nas coordenadas anteriores e novas
          let heading = calculateHeading(previousCoord, newCoord);

          // Ajuste o valor de rotao conforme necessrio
          let rotationAdjustment = -80;
          let correctedHeading = heading + rotationAdjustment;

          // Converter o heading ajustado para radians e criar a orientao do barco
          const hpr = new Cesium.HeadingPitchRoll(
            Cesium.Math.toRadians(correctedHeading), 0, 0
          );

          let lastUpdateTime = null;
          let updateInterval = 500; // Atualiza a linha a cada 500 ms

          function animateBoat() {
            try {
              let currentTime = Date.now();
              let timeElapsed = currentTime - startTime;
              let t = Math.min(timeElapsed / duration, 1); // Progresso da animao

              // Interpolar a posio do barco entre o incio e o fim
              let interpolatedPosition = Cesium.Cartesian3.lerp(
                startPosition,
                endPosition,
                t,
                new Cesium.Cartesian3()
              );

              if (!interpolatedPosition || isNaN(interpolatedPosition.x) || isNaN(interpolatedPosition.y) || isNaN(interpolatedPosition.z)) {
                console.warn('interpolatedPosition  invlido');
                return;
              }

              vehicleEntities[boatName].position = new Cesium.ConstantPositionProperty(interpolatedPosition);

              // Atualizar a orientao do barco conforme ele se move para a nova posio
              vehicleEntities[boatName].orientation = Cesium.Transforms.headingPitchRollQuaternion(
                interpolatedPosition,
                hpr
              );

              // Garantir que polylinePositions[boatName] seja um array vlido
              if (!Array.isArray(polylinePositions[boatName])) {
                polylinePositions[boatName] = [];
              }

              // Adicionar a nova posio  linha amarela em intervalos definidos
              if (!lastUpdateTime || currentTime - lastUpdateTime >= updateInterval) {
                try {
                  polylinePositions[boatName].push(interpolatedPosition.clone());
                } catch (error) {
                  console.warn(`Erro ao atualizar polylinePositions para o barco ${boatName}:`, error);
                }
                lastUpdateTime = currentTime;
              }

              if (t < 1) {
                requestAnimationFrame(animateBoat); // Continua a animao at completar
              } else {
                isMoving[boatName] = false;
                // Adicionar a posio final  linha
                try {
                  polylinePositions[boatName].push(endPosition.clone());
                } catch (error) {
                  console.warn(`Erro ao adicionar posio final  polylinePositions para o barco ${boatName}:`, error);
                }
              }
            } catch (error) {
              console.warn(`Erro na animao do barco ${boatName}:`, error);
            }
          }

          requestAnimationFrame(animateBoat);
        } catch (error) {
          console.warn(`Erro na funo moveBoat para o barco ${boatName}:`, error);
        }
      }

      // Funo para carregar o JSON e simular o movimento contnuo para cada barco
      async function updateBoatPosition(boatName) {
        try {
          const apiUrl = `https://enave.com.br/wp-json/lancha/v1/dados-lancha/?identificacao=${encodeURIComponent(
            boatName
          )}`;
          const response = await fetch(apiUrl);

          // Verifica se a resposta  vlida
          if (!response.ok) {
            console.warn(`Erro ao buscar dados para o barco: ${boatName}. Ignorando esta atualizao.`);
            return; // Ignora esta atualizao
          }

          const data = await response.json();

          // Verifica se os dados esto vazios ou incorretos
          if (!data || data.length === 0) {
            console.warn(`Nenhum dado ou dados incorretos recebidos para o barco: ${boatName}. Ignorando esta atualizao.`);
            return; // Ignora esta atualizao
          }

          console.log(`Dados recebidos para o barco ${boatName}:`, data);

          const newCoordinates = data.slice(0, 2).map((coord) => ({
            lat: parseFloat(coord.latitude),
            lon: parseFloat(coord.longitude),
          }));

          if (!isValidCoord(newCoordinates[0])) {
            console.warn(`Nova coordenada invlida para o barco: ${boatName}`);
            return;
          }

          if (lastProcessedCoords[boatName]) {
            let previousCoord = lastProcessedCoords[boatName];
            let nextCoord = newCoordinates[0];

            if (
              previousCoord.lat !== nextCoord.lat ||
              previousCoord.lon !== nextCoord.lon
            ) {
              moveBoat(previousCoord, nextCoord, boatName);
            }
          }

          lastProcessedCoords[boatName] = newCoordinates[0];

          // Atualizar as coordenadas a cada 10 segundos
          setTimeout(() => updateBoatPosition(boatName), 10000);
        } catch (error) {
          console.warn(
            `Erro ao buscar ou processar dados para o barco: ${boatName}`,
            error
          );
        }
      }

      // Funo para adicionar o barco inicialmente na posio correta
      async function initializeBoat(boatName) {
        try {
          const apiUrl = `https://enave.com.br/wp-json/lancha/v1/dados-lancha/?identificacao=${encodeURIComponent(
            boatName
          )}`;
          const response = await fetch(apiUrl);
          const data = await response.json();

          let initialCoord, nextCoord;

          if (!data || data.length === 0) {
            console.warn(`Nenhum dado recebido para o barco: ${boatName}. O barco ser exibido na posio padro.`);
            initialCoord = defaultCoord;
            nextCoord = defaultCoord;
          } else {
            const initialCoordinates = data.slice(0, 5).map((coord) => ({
              lat: parseFloat(coord.latitude),
              lon: parseFloat(coord.longitude),
            }));

            if (initialCoordinates.length >= 2 && isValidCoord(initialCoordinates[0]) && isValidCoord(initialCoordinates[1])) {
              initialCoord = initialCoordinates[1];
              nextCoord = initialCoordinates[0];
            } else if (initialCoordinates.length >= 1 && isValidCoord(initialCoordinates[0])) {
              initialCoord = initialCoordinates[0];
              nextCoord = initialCoordinates[0]; // Sem movimento inicial
            } else {
              console.warn(`Dados insuficientes ou invlidos para o barco: ${boatName}. O barco ser exibido na posio padro.`);
              initialCoord = defaultCoord;
              nextCoord = defaultCoord;
            }
          }

          if (!isValidCoord(initialCoord) || !isValidCoord(nextCoord)) {
            console.warn(`Coordenadas iniciais invlidas para o barco: ${boatName}. O barco ser exibido na posio padro.`);
            initialCoord = defaultCoord;
            nextCoord = defaultCoord;
          }

          // Armazena a ltima coordenada processada
          lastProcessedCoords[boatName] = nextCoord;

          let heading = calculateHeading(initialCoord, nextCoord);

          const hpr = new Cesium.HeadingPitchRoll(
            Cesium.Math.toRadians(heading - 80), // Ajusta a orientao inicial
            0,
            0
          );
          const startPosition = Cesium.Cartesian3.fromDegrees(
            initialCoord.lon,
            initialCoord.lat
          );

          // Adicionar o barco como uma entidade
          vehicleEntities[boatName] = viewer.entities.add({
            position: new Cesium.ConstantPositionProperty(startPosition),
            model: {
              uri: "<?php echo get_bloginfo('template_directory');?>/barco.glb", // Atualize o caminho para o modelo GLTF
              minimumPixelSize: 50,
              maximumScale: 100,
              heightReference: Cesium.HeightReference.RELATIVE_TO_GROUND,
 
            },
            label: {
              text: boatName,
              font: 'bold 12pt Segoe UI Semibold',
              fillColor: Cesium.Color.YELLOW,
              outlineColor: Cesium.Color.BLACK,
              outlineWidth: 2,
              style: Cesium.LabelStyle.FILL_AND_OUTLINE,
              verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
              pixelOffset: new Cesium.Cartesian2(0, -45),
              heightReference: Cesium.HeightReference.RELATIVE_TO_GROUND,
 
              
            },
            orientation: Cesium.Transforms.headingPitchRollQuaternion(
              startPosition,
              hpr
            ), // Orientao inicial do barco
          });

          polylinePositions[boatName] = [startPosition.clone()];

          polylineEntities[boatName] = viewer.entities.add({
            polyline: {
              positions: new Cesium.CallbackProperty(
                () => polylinePositions[boatName],
                false
              ),
              width: 5,
              material: Cesium.Color.YELLOW.withAlpha(0.7),
              clampToGround: true,
            },
          });

          if (!isFirstBoatInitialized) {
            isFirstBoatInitialized = true;

            viewer.camera.setView({
              destination: startPosition,
              orientation: {
                heading: Cesium.Math.toRadians(90),
                pitch: Cesium.Math.toRadians(-35),
                roll: 0.0,
              },
            });

            setTimeout(() => viewer.camera.zoomOut(18000), 1000);
          }

          // Inicia o processo de atualizao para este barco
          updateBoatPosition(boatName);
        } catch (error) {
          console.warn(`Erro ao inicializar o barco ${boatName}:`, error);
        }
      }

      // Iniciar a aplicao
      async function start() {
        for (let boatName of boatNames) {
          await initializeBoat(boatName);
        }
      }

      start();
    })();
  </script>
</body>
</html>
