 <!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>CesiumJS Barcos com Rastros Contnuos</title>
    <script src="https://cesium.com/downloads/cesiumjs/releases/1.88/Build/Cesium/Cesium.js"></script>
    <link href="https://cesium.com/downloads/cesiumjs/releases/1.88/Build/Cesium/Widgets/widgets.css" rel="stylesheet">
    <style>
        html, body, #cesiumContainer {
            width: 100%;
            height: 100%;
            margin: 0;
            padding: 0;
            overflow: hidden;
        }
    </style>
</head>
<body>
    <div id="cesiumContainer"></div>

    <script>
    Cesium.Ion.defaultAccessToken = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqdGkiOiI1YTI0YTI5Yi1hYmZhLTRlZjQtYmRhOS0xZDYzN2Q2YTE0YjAiLCJpZCI6MjM3MjA0LCJpYXQiOjE3MjQ3MzExNDF9.uyOaFwq9ccT8IX27raWwIs_Fc6glz_UVuTMnogOvszc';

    var viewer = new Cesium.Viewer('cesiumContainer', {
        terrainProvider: Cesium.createWorldTerrain(),
        timeline: false,
        animation: true,
        sceneMode: Cesium.SceneMode.SCENE3D
    });

    var boats = {}; // Armazena os barcos por identificao
    var moveBoatInterval = 5000;
    var duration = 5000;

    // Funo para buscar dados GPS de todos os barcos
    function fetchGPSData() {
        console.log('Fetching GPS data for all boats...');
        return fetch('https://enave.com.br/wp-json/lancha/v1/dados-lancha/')
            .then(response => {
                if (!response.ok) {
                    console.error('Erro ao buscar dados da API:', response.status);
                    return null;
                }
                return response.json();
            })
            .then(data => {
                console.log('Dados recebidos:', data);
                if (data.length >= 10) {
                    // Ordena os dados por data e separa por barco
                    var boatsData = data.reduce((acc, item) => {
                        if (!acc[item.identificacao]) {
                            acc[item.identificacao] = [];
                        }
                        acc[item.identificacao].push({
                            lat: parseFloat(item.latitude),
                            lon: parseFloat(item.longitude),
                            data: item.data
                        });
                        return acc;
                    }, {});
                    console.log('Dados dos barcos organizados:', boatsData);
                    return boatsData;
                } else {
                    console.error('Coordenadas insuficientes recebidas:', data);
                    return null;
                }
            })
            .catch(error => console.error('Erro ao buscar dados da API:', error));
    }

    // Funo para calcular o bearing entre duas coordenadas
    function calculateBearing(start, end) {
        var startLat = Cesium.Math.toRadians(start.lat);
        var startLon = Cesium.Math.toRadians(start.lon);
        var endLat = Cesium.Math.toRadians(end.lat);
        var endLon = Cesium.Math.toRadians(end.lon);

        var dLon = endLon - startLon;
        var y = Math.sin(dLon) * Math.cos(endLat);
        var x = Math.cos(startLat) * Math.sin(endLat) - Math.sin(startLat) * Math.cos(endLat) * Math.cos(dLon);
        var bearing = Math.atan2(y, x);
        return Cesium.Math.toDegrees(bearing);
    }

    // Funo para inicializar o barco e o rastro
    function initializeBoat(boatId, coords) {
        if (coords.length >= 10) {
            // Organiza as coordenadas por ordem cronolgica
            var sortedCoords = coords.sort((a, b) => new Date(b.data) - new Date(a.data));
            var lastPosition = sortedCoords[0];  // ltima coordenada
            var secondLastPosition = sortedCoords[1];  // Penltima coordenada para calcular o bearing
            var bearing = calculateBearing(lastPosition, secondLastPosition);

            console.log('Inicializando barco', boatId, 'na posio:', lastPosition);

            // Adiciona o barco ao mapa
            boats[boatId] = {
                entity: viewer.entities.add({
                    position: Cesium.Cartesian3.fromDegrees(lastPosition.lon, lastPosition.lat, 0),
                    billboard: {
                        image: 'https://enave.com.br/n.png',  // Verifique se a URL da imagem do barco est correta
                        width: 45,
                        height: 45,
                        verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
                        rotation: Cesium.Math.toRadians(90),
                        alignedAxis: Cesium.Cartesian3.UNIT_Z
                    },
                    orientation: Cesium.Transforms.headingPitchRollQuaternion(
                        Cesium.Cartesian3.fromDegrees(lastPosition.lon, lastPosition.lat, 0),
                        new Cesium.HeadingPitchRoll(Cesium.Math.toRadians(bearing), 0, 0)
                    )
                }),
                polylinePositions: sortedCoords.map(coord => Cesium.Cartesian3.fromDegrees(coord.lon, coord.lat, 0)),
                lastCoord: lastPosition
            };

            // Adiciona o rastro do barco
            boats[boatId].polylineEntity = viewer.entities.add({
                polyline: {
                    positions: boats[boatId].polylinePositions,
                    width: 10,
                    material: Cesium.Color.YELLOW.withAlpha(0.7)
                }
            });

            // Inicializa a cmera para o primeiro barco
            if (Object.keys(boats).length === 1) {
                viewer.camera.flyTo({
                    destination: Cesium.Cartesian3.fromDegrees(lastPosition.lon, lastPosition.lat, 5000),
                    duration: 2
                });
            }
        } else {
            console.error('Coordenadas insuficientes para inicializar o barco', boatId, ':', coords);
        }
    }

    // Funo para animar o movimento de todos os barcos
    function fetchAndAnimate() {
        fetchGPSData().then(boatsData => {
            if (boatsData) {
                Object.keys(boatsData).forEach(boatId => {
                    var newCoords = boatsData[boatId];
                    var nextCoord = newCoords[0];  // Coordenada mais recente

                    if (!boats[boatId]) {
                        // Inicializa o barco se ele ainda no foi adicionado
                        initializeBoat(boatId, newCoords);
                    } else {
                        var lastCoord = boats[boatId].lastCoord;
                        if (nextCoord.lat !== lastCoord.lat || nextCoord.lon !== lastCoord.lon) {
                            var previousCoord = lastCoord;
                            boats[boatId].lastCoord = nextCoord;
                            animateBoat(boatId, previousCoord, nextCoord);
                        }
                    }
                });
            } else {
                console.error('Nenhum dado de barcos recebido.');
            }

            // Continuar a buscar novas coordenadas aps o intervalo definido
            setTimeout(fetchAndAnimate, moveBoatInterval);
        });
    }

    // Funo para animar o movimento de um barco
    function animateBoat(boatId, previousCoord, newCoord) {
        var startTime = Date.now();

        function moveBoat() {
            var currentTime = Date.now();
            var timeElapsed = currentTime - startTime;
            var t = timeElapsed / duration;
            if (t >= 1) t = 1;

            var interpolatedLon = Cesium.Math.lerp(previousCoord.lon, newCoord.lon, t);
            var interpolatedLat = Cesium.Math.lerp(previousCoord.lat, newCoord.lat, t);
            var newPosition = Cesium.Cartesian3.fromDegrees(interpolatedLon, interpolatedLat, 0);

            // Atualiza a posio do barco
            boats[boatId].entity.position = newPosition;

            // Adiciona a nova posio ao rastro
            boats[boatId].polylinePositions.push(newPosition);
            boats[boatId].polylineEntity.polyline.positions = boats[boatId].polylinePositions;

            var bearing = calculateBearing(previousCoord, newCoord);
            boats[boatId].entity.orientation = Cesium.Transforms.headingPitchRollQuaternion(
                newPosition,
                new Cesium.HeadingPitchRoll(Cesium.Math.toRadians(bearing), 0, 0)
            );

            if (t < 1) {
                requestAnimationFrame(moveBoat);
            }
        }

        requestAnimationFrame(moveBoat);
    }

    // Inicializa os barcos e animao
    fetchGPSData().then(boatsData => {
        if (boatsData) {
            Object.keys(boatsData).forEach(boatId => {
                initializeBoat(boatId, boatsData[boatId]);
            });
            setTimeout(fetchAndAnimate, moveBoatInterval);
        } else {
            console.error('Erro ao inicializar barcos. Dados invlidos:', boatsData);
        }
    });
    </script>
</body>
</html>


  