<?php
header('Content-Type: application/json');

date_default_timezone_set('America/Manaus');
ini_set('display_errors', 1);
error_reporting(E_ALL);

$identificacao = isset($_REQUEST['identificacao']) ? $_REQUEST['identificacao'] : '';

if (empty($identificacao)) {
    echo json_encode(['error' => 'Identificao no fornecida.']);
    exit;
}

// Verifica se a funo WP_Query existe
if (!class_exists('WP_Query')) {
    echo json_encode(['error' => 'WP_Query no disponvel.']);
    exit;
}

// Define the arguments for the query
$args = array(
    'post_type' => 'dados_lancha',
    'posts_per_page' => 40,
    'order' => 'DESC',
    'meta_query' => array(
        array(
            'key' => 'identificacao',
            'value' => $identificacao,
            'compare' => '=',
        ),
    ),
);

$qry = new WP_Query($args);

if (!$qry->have_posts()) {
    echo json_encode(['error' => 'Nenhum dado encontrado para a identificao fornecida.']);
    exit;
}

$posts = array_reverse($qry->get_posts());

$coords = [];
$info = [];
foreach ($posts as $post) {
    setup_postdata($post);

    $lat = get_post_meta($post->ID, 'latitude', true);
    $lng = get_post_meta($post->ID, 'longitude', true);
    $datah = get_post_meta($post->ID, 'datah', true);

    if (!$lat || !$lng || !$datah) {
        continue; // Pular posts sem dados completos
    }

    $coords[] = ['lat' => floatval($lat), 'lng' => floatval($lng)];
    $info[] = [
        'label' => 'DataHora',
        'value' => date("d/m/Y H:i:s", strtotime($datah))
    ];
}
wp_reset_postdata();

if (empty($coords)) {
    echo json_encode(['error' => 'Nenhuma coordenada vlida encontrada.']);
    exit;
}

echo json_encode(['pathCoords' => $coords, 'info' => $info]);
exit;
?>
